#!/usr/bin/env python

'''
by OH2GBA 

Simple script to lookup a domain and patch an ini file

24th of April 2017
v0.1 initial version

25th of April 2017
v0.2 Allow to pass a static IP address

26th of April 2017
v0.3 Changed backup file name

'''

import socket
import unittest
import shutil
import os
import sys
import time


# -------- CONFIGURATION --------

#Update this variable
VARIABLE = 'sdr_ip_address'

#in this file
CONFIG_FN = 'sdr_config.ini'

#filename used as backup
config_fn_backup = 'sdr_config_backup.ini'

#log file
LOG_FN='sdr_config.log'

#Delete the log at every start
DELETE_LOG_AT_START = True

#Line end used in the config file
LINE_END = "\n"

#Script version
VERSION="0.3"

# -------- START OF CODE --------



def delete_log(log_fn=LOG_FN):
    '''attempt to delete the log file'''
    try:
        os.remove(log_fn)
    except:
        pass

def log(txt):
    '''simple logging'''
    t = time.strftime("%d %b %Y %H:%M:%S ", time.localtime())
    try:
        with open(LOG_FN, 'a') as _file:
            _file.write(t + str(txt) + "\r\n")
    except Exception as err:
        print '''failed to write the log file %s'''%(err)
        
def make_backup(src=None, dst=None):
    '''make a backup of the configuration, return True if success'''
    try:
        if src is None:
            src = CONFIG_FN
        if dst is None:
            dst = config_fn_backup
        if not os.path.isfile(src):
            raise Exception("src file: %s does not exist"%(src))
        shutil.copy(src, dst)
        log("Backup made")
        return True
    except Exception as err:
        log("Could not make a backup file: %s"%(err))
        return False
    
def set_variable(variable, value, src, dst):
    '''Replace value of variable'''
    success = False
    log("Set variable: %s, to value: %s"%(variable, value))
    with open(src, 'r') as _src:
        _src = _src.read()
        _src = _src.split(LINE_END)
        with open(dst, 'w') as _dst:
            for line_nr, new_line in enumerate(_src):
                written = False
                try:
                    line = new_line.split("=")
                    if len(line) == 2:
                        if variable in line[0]:
                            log("""'%s' found in the log file at line: %s"""%(variable, line_nr))
                            line[1] = value
                            line = "=".join(line)
                            _dst.write(str(line) + LINE_END)
                            success = True
                            written = True
                except Exception as err:
                    log('ignore line %s in %s (%s)'%(line_nr, src, err))
                    
                if not written:
                    try:
                        if len(new_line) > 0:
                            _dst.write(str(new_line) + LINE_END)
                    except Exception as err:
                        log("Could not write %s: %s"%(new_line, err))
                    
    if not success:
        log("Variable %s not set to %s (variable not in config?)"%(variable, value))
    return success

    

def gethostbyname(hostname):
    '''
    return the ip address of a host. 
    example: 
    gethostbyname("google.com")
    Out[2]: '172.217.22.174'

    return None when either an error or host not found

    '''
    retval = None
    try:
        log("Lookup: %s"%(hostname))
        retval = socket.gethostbyname(hostname)
        log("Found at ip: %s"%(retval))
    except Exception as err:
        log("Could not lookup ip: %s"%(err))
    return retval

def check_ip(ip):
    '''return True if ip is a valid ip, otherwise False'''
    retval = False

    try:
        socket.inet_aton(ip)
        retval = True
    except Exception:
        pass

    return retval

class Testcases(unittest.TestCase):

    def test_good_host(self):
        tests = ['google.com', 'www.google.com', 'www.google.com']
        for test in tests:
            ip = gethostbyname(test)
            self.assertTrue(check_ip(ip))

    def test_bad_host(self):
        tests = ['foobar', 'http:\\www.google.com', 'https:\\google.com']
        for test in tests:
            ip = gethostbyname(test)
            self.assertFalse(check_ip(ip))
    
    def test_backup(self):
        self.assertTrue(make_backup())
        self.assertFalse(make_backup(src='foo'))
    
    def test_setvariable(self):
        self.assertFalse(set_variable('foo', 'bar', CONFIG_FN, config_fn_backup))
        self.assertTrue(set_variable('sdr_ip_address', 'bar', config_fn_backup, CONFIG_FN))

def run(arg):
    '''main program'''
    start_t = time.time()
    if DELETE_LOG_AT_START:
        delete_log()
    log("%s version %s"%(sys.argv[0], VERSION))
    host = None
    #test if the given host is a valid ip addres
    if check_ip(arg):
       log("Argument is an IP address, using this")
       host = arg
    else:
        ip = gethostbyname(arg)
        if ip != None:
            if check_ip(ip):
                host = ip
    
    if host != None:
        if make_backup(CONFIG_FN, config_fn_backup):
            set_variable(VARIABLE, host, config_fn_backup, CONFIG_FN)
    else:
        log("Invalid argument")
        
    delta = time.time() - start_t
    delta = int(delta * 1000)
    log("Operation took %sms"%(delta))

if __name__ == '__main__':
    if len(sys.argv) == 1:
        print "usage: python %s <hostname>"%(sys.argv[0])
    if len(sys.argv) > 1:
        if 'test' in sys.argv[1]:
            sys.argv = [sys.argv[0]]
            unittest.main()
        else:
            run(sys.argv[1])



